import { OnModuleDestroy, OnModuleInit } from '@nestjs/common';
import { IEvent, IEventPublisher, EventBus, CommandBus, QueryBus, IMessageSource } from '@nestjs/cqrs';
import { ExplorerService } from '@nestjs/cqrs/dist/services/explorer.service';
import * as Long from 'long';
import { Subject } from 'rxjs';
import { EventStorePersistentSubscription, ResolvedEvent, EventStoreCatchUpSubscription, EventStoreSubscription as EventStoreVolatileSubscription } from 'node-eventstore-client';
import { EventStoreOptionConfig, IEventConstructors, EventStorePersistentSubscription as ESPersistentSubscription, EventStoreCatchupSubscription as ESCatchUpSubscription, EventStoreVolatileSubscription as ESVolatileSubscription, ExtendedCatchUpSubscription, ExtendedVolatileSubscription, ExtendedPersistentSubscription } from './contract/event-store-option.config';
import { IEventStoreConnectConfig } from './contract/event-store-connect-config.interface';
export declare class EventStore implements IEventPublisher, OnModuleDestroy, OnModuleInit, IMessageSource {
    private readonly explorerService;
    private readonly eventsBus;
    private readonly commandsBus;
    private readonly queryBus;
    private logger;
    private eventStore;
    private eventHandlers;
    private subject$;
    private readonly featureStream?;
    private catchupSubscriptions;
    private catchupSubscriptionsCount;
    private persistentSubscriptions;
    private persistentSubscriptionsCount;
    private volatileSubscriptions;
    private volatileSubscriptionsCount;
    constructor(eventStore: any, configService: IEventStoreConnectConfig, esStreamConfig: EventStoreOptionConfig, explorerService: ExplorerService, eventsBus: EventBus, commandsBus: CommandBus, queryBus: QueryBus);
    publish(event: IEvent, stream?: string): Promise<void>;
    subscribeToPersistentSubscriptions(subscriptions: ESPersistentSubscription[]): Promise<void>;
    subscribeToCatchUpSubscriptions(subscriptions: ESCatchUpSubscription[]): void;
    subscribeToVolatileSubscriptions(subscriptions: ESVolatileSubscription[]): Promise<void>;
    subscribeToCatchupSubscription(stream: string, resolveLinkTos?: boolean, lastCheckpoint?: number | Long | null): ExtendedCatchUpSubscription;
    subscribeToVolatileSubscription(stream: string, resolveLinkTos?: boolean): Promise<ExtendedVolatileSubscription>;
    readonly allCatchUpSubscriptionsLive: boolean;
    readonly allVolatileSubscriptionsLive: boolean;
    readonly allPersistentSubscriptionsLive: boolean;
    subscribeToPersistentSubscription(stream: string, subscriptionName: string): Promise<ExtendedPersistentSubscription>;
    onEvent(_subscription: EventStorePersistentSubscription | EventStoreCatchUpSubscription | EventStoreVolatileSubscription, payload: ResolvedEvent): Promise<void>;
    onDropped(subscription: ExtendedPersistentSubscription | ExtendedCatchUpSubscription | ExtendedVolatileSubscription, _reason: string, error: Error): void;
    onLiveProcessingStarted(subscription: ExtendedCatchUpSubscription): void;
    readonly isLive: boolean;
    addEventHandlers(eventHandlers: IEventConstructors): void;
    onModuleInit(): any;
    onModuleDestroy(): any;
    bridgeEventsTo<T extends IEvent>(subject: Subject<T>): Promise<any>;
}
