"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const cqrs_1 = require("@nestjs/cqrs");
const explorer_service_1 = require("@nestjs/cqrs/dist/services/explorer.service");
const uuid_1 = require("uuid");
const node_eventstore_client_1 = require("node-eventstore-client");
const event_store_option_config_1 = require("./contract/event-store-option.config");
const nestjs_event_store_constant_1 = require("./contract/nestjs-event-store.constant");
let EventStore = class EventStore {
    constructor(eventStore, configService, esStreamConfig, explorerService, eventsBus, commandsBus, queryBus) {
        this.explorerService = explorerService;
        this.eventsBus = eventsBus;
        this.commandsBus = commandsBus;
        this.queryBus = queryBus;
        this.logger = new common_1.Logger(this.constructor.name);
        this.catchupSubscriptions = [];
        this.persistentSubscriptions = [];
        this.volatileSubscriptions = [];
        this.eventStore = eventStore;
        this.featureStream = esStreamConfig.featureStreamName;
        this.addEventHandlers(esStreamConfig.eventHandlers);
        this.eventStore.connect(configService.options, configService.tcpEndpoint);
        const catchupSubscriptions = esStreamConfig.subscriptions.filter((sub) => {
            return sub.type === event_store_option_config_1.EventStoreSubscriptionType.CatchUp;
        });
        const persistentSubscriptions = esStreamConfig.subscriptions.filter((sub) => {
            return sub.type === event_store_option_config_1.EventStoreSubscriptionType.Persistent;
        });
        const volatileSubscriptions = esStreamConfig.subscriptions.filter((sub) => {
            return sub.type === event_store_option_config_1.EventStoreSubscriptionType.Volatile;
        });
        this.subscribeToCatchUpSubscriptions(catchupSubscriptions);
        this.subscribeToPersistentSubscriptions(persistentSubscriptions);
        this.subscribeToVolatileSubscriptions(volatileSubscriptions);
    }
    async publish(event, stream) {
        if (event === undefined) {
            return;
        }
        if (event === null) {
            return;
        }
        const eventPayload = node_eventstore_client_1.createJsonEventData(uuid_1.v4(), event, null, stream);
        const streamId = stream ? stream : this.featureStream;
        try {
            await this.eventStore.getConnection().appendToStream(streamId, node_eventstore_client_1.expectedVersion.any, [eventPayload]);
        }
        catch (err) {
            this.logger.error(err);
        }
    }
    async subscribeToPersistentSubscriptions(subscriptions) {
        this.persistentSubscriptionsCount = subscriptions.length;
        this.persistentSubscriptions = await Promise.all(subscriptions.map(async (subscription) => {
            return await this.subscribeToPersistentSubscription(subscription.stream, subscription.persistentSubscriptionName);
        }));
    }
    subscribeToCatchUpSubscriptions(subscriptions) {
        this.catchupSubscriptionsCount = subscriptions.length;
        this.catchupSubscriptions = subscriptions.map((subscription) => {
            return this.subscribeToCatchupSubscription(subscription.stream, subscription.resolveLinkTos, subscription.lastCheckpoint);
        });
    }
    async subscribeToVolatileSubscriptions(subscriptions) {
        this.volatileSubscriptionsCount = subscriptions.length;
        this.volatileSubscriptions = await Promise.all(subscriptions.map(async (subscription) => {
            return await this.subscribeToVolatileSubscription(subscription.stream, subscription.resolveLinkTos);
        }));
    }
    subscribeToCatchupSubscription(stream, resolveLinkTos = true, lastCheckpoint = 0) {
        this.logger.log(`Catching up and subscribing to stream ${stream}!`);
        try {
            return this.eventStore.getConnection().subscribeToStreamFrom(stream, lastCheckpoint, resolveLinkTos, (sub, payload) => this.onEvent(sub, payload), subscription => this.onLiveProcessingStarted(subscription), (sub, reason, error) => this.onDropped(sub, reason, error));
        }
        catch (err) {
            this.logger.error(err.message);
        }
    }
    async subscribeToVolatileSubscription(stream, resolveLinkTos = true) {
        this.logger.log(`Volatile and subscribing to stream ${stream}!`);
        try {
            const resolved = await this.eventStore.getConnection().subscribeToStream(stream, resolveLinkTos, (sub, payload) => this.onEvent(sub, payload), (sub, reason, error) => this.onDropped(sub, reason, error));
            this.logger.log('Volatile processing of EventStore events started!');
            resolved.isLive = true;
            return resolved;
        }
        catch (err) {
            this.logger.error(err.message);
        }
    }
    get allCatchUpSubscriptionsLive() {
        const initialized = this.catchupSubscriptions.length === this.catchupSubscriptionsCount;
        return (initialized &&
            this.catchupSubscriptions.every((subscription) => {
                return !!subscription && subscription.isLive;
            }));
    }
    get allVolatileSubscriptionsLive() {
        const initialized = this.volatileSubscriptions.length === this.volatileSubscriptionsCount;
        return (initialized &&
            this.volatileSubscriptions.every((subscription) => {
                return !!subscription && subscription.isLive;
            }));
    }
    get allPersistentSubscriptionsLive() {
        const initialized = this.persistentSubscriptions.length === this.persistentSubscriptionsCount;
        return (initialized &&
            this.persistentSubscriptions.every((subscription) => {
                return !!subscription && subscription.isLive;
            }));
    }
    async subscribeToPersistentSubscription(stream, subscriptionName) {
        try {
            this.logger.log(`
       Connecting to persistent subscription ${subscriptionName} on stream ${stream}!
      `);
            const resolved = await this.eventStore.getConnection().connectToPersistentSubscription(stream, subscriptionName, (sub, payload) => this.onEvent(sub, payload), (sub, reason, error) => this.onDropped(sub, reason, error));
            resolved.isLive = true;
            return resolved;
        }
        catch (err) {
            this.logger.error(err.message);
        }
    }
    async onEvent(_subscription, payload) {
        const { event } = payload;
        if (!event || !event.isJson) {
            this.logger.error('Received event that could not be resolved!');
            return;
        }
        const handler = this.eventHandlers[event.eventType];
        if (!handler) {
            this.logger.error('Received event that could not be handled!');
            return;
        }
        const rawData = JSON.parse(event.data.toString());
        const data = Object.values(rawData);
        const eventType = event.eventType || rawData.content.eventType;
        if (this.eventHandlers && this.eventHandlers[eventType]) {
            this.subject$.next(this.eventHandlers[event.eventType](...data));
        }
        else {
            common_1.Logger.warn(`Event of type ${eventType} not handled`, this.constructor.name);
        }
    }
    onDropped(subscription, _reason, error) {
        subscription.isLive = false;
        this.logger.error(error);
    }
    onLiveProcessingStarted(subscription) {
        subscription.isLive = true;
        this.logger.log('Live processing of EventStore events started!');
    }
    get isLive() {
        return (this.allCatchUpSubscriptionsLive && this.allPersistentSubscriptionsLive && this.allVolatileSubscriptionsLive);
    }
    addEventHandlers(eventHandlers) {
        this.eventHandlers = Object.assign(Object.assign({}, this.eventHandlers), eventHandlers);
    }
    onModuleInit() {
        const { events, queries, sagas, commands } = this.explorerService.explore();
        this.eventsBus.register(events);
        this.commandsBus.register(commands);
        this.queryBus.register(queries);
        this.eventsBus.registerSagas(sagas);
        this.subject$ = this.eventsBus.subject$;
        this.bridgeEventsTo(this.eventsBus.subject$);
        this.eventsBus.publisher = this;
    }
    onModuleDestroy() {
        this.eventStore.close();
    }
    async bridgeEventsTo(subject) {
        this.subject$ = subject;
    }
};
EventStore = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(nestjs_event_store_constant_1.ProvidersConstants.EVENT_STORE_PROVIDER)),
    __param(1, common_1.Inject(nestjs_event_store_constant_1.ProvidersConstants.EVENT_STORE_CONNECTION_CONFIG_PROVIDER)),
    __param(2, common_1.Inject(nestjs_event_store_constant_1.ProvidersConstants.EVENT_STORE_STREAM_CONFIG_PROVIDER)),
    __metadata("design:paramtypes", [Object, Object, Object, explorer_service_1.ExplorerService,
        cqrs_1.EventBus,
        cqrs_1.CommandBus,
        cqrs_1.QueryBus])
], EventStore);
exports.EventStore = EventStore;
//# sourceMappingURL=event-store.js.map