import { Logger } from '@nestjs/common';
import assert from 'assert';
import uuid from 'uuid';
import { createConnection } from 'node-eventstore-client';
export class NestjsEventStore {
    constructor() {
        this.logger = new Logger(this.constructor.name);
        this.type = 'event-store';
    }
    connect(options, endpoint) {
        try {
            this.connection = createConnection(options, endpoint);
            this.connection.connect();
            this.connection.on('connected', () => {
                this.isConnected = true;
                this.logger.log('EventStore connected!');
            });
            this.connection.on('closed', () => {
                this.isConnected = false;
                this.logger.error('EventStore closed!');
                this.connect(options, endpoint);
            });
            return this;
        }
        catch (e) {
            this.logger.error(e.message);
            throw new Error(e);
        }
    }
    getConnection() {
        return this.connection;
    }
    newEvent(name, payload) {
        return this.newEventBuilder(name, payload);
    }
    newEventBuilder(eventType, data, metadata, eventId) {
        assert(eventType);
        assert(data);
        const event = {
            eventId: eventId || uuid.v4(),
            eventType,
            data
        };
        if (metadata !== undefined) {
            event.metadata = metadata;
        }
        return event;
    }
    close() {
        this.connection.close();
        return this;
    }
}
//# sourceMappingURL=nestjs-event-store.class.js.map